package fr.eurecom.ratcom.multicast;

import sun.net.*;

import java.io.IOException;
import java.net.*;

import fr.eurecom.ratcom.properties.HandleProperties;
import fr.eurecom.ratcom.properties.ProjectProperties;
import fr.eurecom.ratcom.util.Logging;


/**
 * Class to handle Multicast related issues for server and client sides of the
 * application.
 * 
 * @author camara
 * 
 */
public class Multicast {

	public static boolean OK = true;
	public static boolean PROBLEM = false;
	public static int SERVER = 0;
	public static int CLIENT = 1;
	
	private int multicastType = SERVER;
	ProjectProperties properties = ProjectProperties.getInstance();
	MulticastSocket multicastSocket = null;
	static Logging logging = Logging.getLogger();// ("smsLog.txt");

	/**
     * Singleton class, this means that all classes
     * that ask for Multicast connection will always receive the same instance
     * of the class, thus will share the same information
     */    
	public Multicast() {
		try {
			multicastSocket = new MulticastSocket(properties.multicastPort);
			multicastSocket.setTimeToLive(properties.multicastTTL);
			multicastSocket.joinGroup(InetAddress
					.getByName(properties.multicastGroup));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	/**
	 * Creates the socket for the client multicast
	 * 
	 * @return OK if everything is fine and PROBLEM otherwise
	 */
	public boolean createClient() {
//		try {
//			multicastSocket.joinGroup(InetAddress
//					.getByName(properties.multicastGroup));
			multicastType = CLIENT; 
//		} catch (IOException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//			return PROBLEM;
//		}
		return OK;
	}

	/**
	 * Receives the data multicast
	 * 
	 * @return PROBLEM when the socket is removed and a data is received
	 */
	public String clientReceive() {

		byte buf[] = new byte[1024];
		DatagramPacket pack = new DatagramPacket(buf, buf.length);
		try {
			multicastSocket.receive(pack);
			logging.logln("Multicast: Client received Ok");
		} catch (IOException e) {
			logging.logln("Multicast: Client received problem -> " + e.getMessage());
			// the socket is not working anymore
			e.printStackTrace();
			RCV_PROBLEM = true;
			return "";
		}
		return new String(pack.getData());
	}
	
	private boolean RCV_PROBLEM = false;

	/**
	 * Prints the data multicast stream
	 * 
	 * @return PROBLEM when the socket is removed and a data is received
	 */
	public void clientReceivePrint() {

		String message=null;
		// receives the data forever, or at least until the multicastSocket is
		// up
		while (!RCV_PROBLEM) {
			message =  clientReceive(); 
//			System.out.println("Received data from: "
//					+ pack.getAddress().toString() + ":" + pack.getPort()
//					+ " with length: " + pack.getLength());
//			String message = pack.getData().toString();
			System.out.println(message);
		}
	}

	/**
	 * Sends a multicast message
	 * 
	 * @param msg
	 *            : the text of the message to send
	 * @return OK if everything is fine and PROBLEM otherwise
	 */
	public boolean serverSend(String msg) {

//		byte buf[] = new byte[10];
//		for (int i = 0; i < buf.length; i++)
//			buf[i] = (byte) i;
		logging.logln("Multicast: Sending msg: "+ msg);

		// Create a DatagramPacket
		DatagramPacket pack;
		try {
			InetAddress group = InetAddress
					.getByName(properties.multicastGroup);
			DatagramPacket packet = new DatagramPacket(msg.getBytes(), msg
					.length(), group, properties.multicastPort);
			multicastSocket.send(packet); 
			logging.logln("Multicast: sent Oks");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			logging.logln("Multicast: sent problem -> " + e.getMessage());
			e.printStackTrace();
			return PROBLEM;
		}
		return OK;
	}

	/**
	 * Stop the multicast socket. Removes the client from the group and closes
	 * the socket.
	 * 
	 * @return OK if everything was fine, and PROBLEM if not
	 */
	public boolean stopMulticast() {
		try {
			// only the client registers to the traffic the server don't
			if (multicastType == CLIENT)
				multicastSocket.leaveGroup(InetAddress
						.getByName(properties.multicastGroup));
			multicastSocket.close();
			logging.logln("Multicast stopped");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return PROBLEM;
		}
		return OK;

	}

	/**
	 * Used only for test purposes
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub

	}


}
