package fr.eurecom.ratcom.vrsu;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketAddress;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.Enumeration;
import java.util.HashSet;

import fr.eurecom.ratcom.properties.ProjectProperties;
import fr.eurecom.ratcom.util.Logging;
import fr.eurecom.ratcom.util.StringHandler;

public class VirtualRoadSideUnitNet {
	private static ProjectProperties properties = ProjectProperties
			.getInstance();
	private static String separator = properties.ID_Separator;
	static Logging logging = Logging.getLogger();// ("smsLog.txt");
	static DatagramSocket vRSUSocket = null;

	/**
	 * Sends a broadcast vRSU alert message
	 * 
	 * @param message
	 * @throws SocketException
	 */
	public static void sendMessage(String message, int ID)
			throws SocketException {
		logging.logln("vRSU sending msg " + ID);

		DatagramSocket tmpSocket = new DatagramSocket();
		tmpSocket.setBroadcast(true);

		// add the ID of the message in the beginning of the message
		message = properties.vRSU_Identifier + separator + ID + separator
				+ message;
		try {
			InetAddress address;
			address = InetAddress.getByName(properties.vRSU_BroadcastIP);
			DatagramPacket warningMessagePkt = new DatagramPacket(message
					.getBytes(), message.getBytes().length, address,
					properties.vRSU_Port);
			tmpSocket.connect(address, properties.vRSU_Port);

			tmpSocket.send(warningMessagePkt);
			logging.logln("vRSU sent msg " + ID);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	/**
	 * Sends a broadcast vRSU alert message
	 * 
	 * @param message
	 * @throws SocketException
	 */
	public static void sendOverAllInterfaces(String message, int ID)
			throws SocketException {
		logging.logln("vRSU sending ALL " + ID);

		// add the ID of the message in the beginning of the message
		message = properties.vRSU_Identifier + separator + ID + separator
				+ message;

		int netMask = 0xffffff00; // assumes a class D subnet change if needed
		int broadcastAddr = netMask ^ 0xffffffff; // subnet broadcast address

		DatagramSocket tmpSocket = new DatagramSocket();
		tmpSocket.setBroadcast(true);
		Enumeration<NetworkInterface> interfaces = NetworkInterface
				.getNetworkInterfaces();

		// for all known interfaces pay attention that when sending a 
		// broadcast, if you are listening the same port Could also
		// receive the packet, so it is better to put a filter in the reception
		while (interfaces.hasMoreElements()) {
			NetworkInterface netInter = interfaces.nextElement();
			Enumeration<InetAddress> addresses = netInter.getInetAddresses();

			// for all the addresses attached to this specific interface
			while (addresses.hasMoreElements()) {
				InetAddress inetAddr = addresses.nextElement();

				// for now works only of Ipv4 addresses and ignores loop back
				if ((inetAddr instanceof Inet4Address)
						&& inetAddr.getAddress()[0] != (byte) 127) {
					// Generates the broadcast address
					byte[] addr = inetAddr.getAddress();
					for (int i = 3; i >= 0; i--) {
						addr[i] |= (byte) (broadcastAddr >>> (3 - i) * 8);
					}

					// sends the message to the given broadcast address
					InetAddress bcAddr = null;
					try {
						bcAddr = InetAddress.getByAddress(addr);
						DatagramPacket warningMessagePkt = new DatagramPacket(
								message.getBytes(), message.getBytes().length,
								bcAddr, properties.vRSU_Port);
						tmpSocket.send(warningMessagePkt);
					} catch (Exception e) {
						logging.logln("vRSU sending ALL problem -> "
								+ e.getMessage());
						logging.logln("vRSU sending ALL Couldn't send to: "
								+ bcAddr.getHostAddress());
						e.printStackTrace();
					}
				}
			}
		}
	}

	private static HashSet<Integer> receivedMessages = new HashSet<Integer>();

	/**
	 * Receives a broadcast message, returns only if receive a NEW vRSU alert
	 * message
	 * 
	 * @param message
	 * @throws SocketException
	 */
	public static String receiveMessage() throws SocketException {
		byte[] message = new byte[1024];
		boolean newMessage = false;

		if (vRSUSocket == null) {
			vRSUSocket = new DatagramSocket(properties.vRSU_Port);
			vRSUSocket.setBroadcast(true);
		}
		try {
			while (!newMessage) {
				InetAddress address;
				address = InetAddress.getByName(properties.vRSU_BroadcastIP);
				DatagramPacket warningMessagePkt = new DatagramPacket(message,
						message.length, address, properties.vRSU_Port);
				vRSUSocket.receive(warningMessagePkt);
				
//				SocketAddress sentAddress=warningMessagePkt.getSocketAddress();
				
				message = warningMessagePkt.getData();

				// verifies if have already received the message
				newMessage = isNewVrsuMessage(message);
				logging.logln("vRSU received msg, vRSU= " + newMessage
						+ " msg:" + new String(message));
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		}
		return new String(message);
	}

	/**
	 * Verifies if the message is a vRSU message and if so, if it is a new
	 * message if it is a new message registers it at the set of received
	 * messages
	 * 
	 * @param message
	 * @param newMessage
	 * @return
	 */
	private static boolean isNewVrsuMessage(byte[] message) {
		String[] splitedMessage = StringHandler.splitedString(message,
				separator);
		if (splitedMessage.length > 2
				&& splitedMessage[0].equals(properties.vRSU_Identifier)) {
			try {
				Integer id = Integer.parseInt(splitedMessage[1]);
				if (!receivedMessages.contains(id)) {
					receivedMessages.add(id);
					return true;
				}

			} catch (NumberFormatException e) {
			}

		}
		return false;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		VirtualRoadSideUnitNet vrs = new VirtualRoadSideUnitNet();
		ReceiveTestThread rt = vrs.new ReceiveTestThread();
		rt.start();
		try {
			VirtualRoadSideUnitNet.sendOverAllInterfaces("Message", 1);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	public class ReceiveTestThread extends Thread {
	
		public void run() {
			try {
				String s = receiveMessage();
			} catch (SocketException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

}
