/**
 * Ratcom <br>
 * Copyright (c) 2010,  <br>
 * All rights reserved.<br>
 * The authors reserve all rights over the content of this file<br>
 * Any use\copy\alteration of it must be authorized by the authors<br> 
 * 
 * @date Dec 6, 2010 - 9:58:39 PM 
 * @author Daniel Camara - danielcamara@gmail.com
 */
package fr.eurecom.ratcom.xml;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;

import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import fr.eurecom.ratcom.properties.ProjectProperties;
import fr.eurecom.ratcom.util.FileHandler;

/**
 * Ratcom <br>
 * 
 * @file fr.eurecom.ratcom.xml.XMLAlertParser.java
 * @author daniel
 * @date Dec 6, 2010 - 9:58:39 PM
 */
public class XMLAlertParser {

	public static int lastMsgID = 0;

	private static ProjectProperties properties = ProjectProperties
			.getInstance();

	public static Schema getSchema(String fileName) {
		SchemaFactory factory = SchemaFactory
				.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
		Schema schema = null;
		try {
			File schemaFile = new File(fileName);
			schema = factory.newSchema(schemaFile);
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		// Validator validator = schema.newValidator();
		return schema;
	}

	public static Document getDocument(String xmlString, Schema schema) {
		DocumentBuilder parser;
		Document document = null;

		try {
			InputStream is = new ByteArrayInputStream(xmlString
					.getBytes("UTF-8"));
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			dbf.setNamespaceAware(true);
			parser = dbf.newDocumentBuilder();
			document = parser.parse(is);
			Validator validator = schema.newValidator();
			validator.validate(new DOMSource(document));
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return document;
	}

	/**
	 * Get the string value of an element of a DOM document
	 * 
	 * @param document
	 * @param expression
	 *            address of the element to read
	 * @return the value of the element
	 * 
	 * @date Dec 6, 2010 - 10:25:27 PM : daniel
	 */
	public static String getValue(Document document, String expression) {
		String returnValue = "";
		XPath xpath = XPathFactory.newInstance().newXPath();
		// String expression = "/birthdate/year";

		Node node;
		try {
			// First, obtain the element as a node.
			node = (Node) xpath.evaluate(expression, document,
					XPathConstants.NODE);
			// Next, obtain the element as a String.

			returnValue = (String) xpath.evaluate(expression, document,
					XPathConstants.STRING);
		} catch (XPathExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return returnValue;
	}

	/**
	 * Get the string value of an element of a DOM document
	 * 
	 * @param document
	 * @param expression
	 *            address of the element to read
	 * @return the value of the element
	 * 
	 * @date Dec 6, 2010 - 10:25:27 PM : daniel
	 */
	public static String getDataSet(Document document, String mainKey,
			String secondaryKeys[], String labels[]) {
		String returnValue = "";
		XPath xPath = XPathFactory.newInstance().newXPath();
		// String expression = "/birthdate/year";

		NodeList list = null;
		try {
			// First, obtain the element as a node.
			list = (NodeList) xPath.evaluate(mainKey, document,
					XPathConstants.NODESET);
			int nNodes = list.getLength();
			for (int i = 0; i < nNodes; i++) {
				Element show = (Element) list.item(i);

				for (int j = 0; j < secondaryKeys.length; j++) {
					returnValue += labels[j]
							+ xPath.evaluate(secondaryKeys[j], show);
				}
				returnValue += properties.newline;
			}

			// Next, obtain the element as a String.
			//
			// returnValue = (String) xpath.evaluate(expression, document,
			// XPathConstants.STRING);
		} catch (XPathExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return returnValue;
	}

	/**
	 * @param args
	 * 
	 * @date Dec 6, 2010 - 9:58:39 PM : daniel
	 */
	public static void main(String[] args) {
		// String xmlSchema = FileHandler.readFile("alertSchema.xsd");
		Schema schema = XMLAlertParser.getSchema(properties.xmlSchemaFile);

		System.out.println("*** " + properties.xmlTestFile1
				+ properties.newline);
		String xmlString = FileHandler.readFileUTF8(properties.xmlTestFile1);
		String msg = XMLAlertParser.getMessageToShow(xmlString);
		System.out.println(msg + properties.newline
				+ "***************************" + properties.newline
				+ properties.newline);

		System.out.println("*** " + properties.xmlTestFile2
				+ properties.newline);
		xmlString = FileHandler.readFileUTF8(properties.xmlTestFile2);
		msg = XMLAlertParser.getMessageToShow(xmlString);
		System.out.println(msg + properties.newline
				+ "***************************" + properties.newline
				+ properties.newline);

		System.out.println("*** " + properties.xmlTestFile3
				+ properties.newline);
		xmlString = FileHandler.readFileUTF8(properties.xmlTestFile3);
		msg = XMLAlertParser.getMessageToShow(xmlString);
		System.out.println(msg + properties.newline
				+ "***************************" + properties.newline
				+ properties.newline);

		System.out.println("*** " + properties.xmlTestFile4
				+ properties.newline);
		xmlString = FileHandler.readFileUTF8(properties.xmlTestFile4);
		msg = XMLAlertParser.getMessageToShow(xmlString);
		System.out.println(msg + properties.newline
				+ "***************************" + properties.newline
				+ properties.newline);

		System.out.println("*** " + properties.xmlTestFile5
				+ properties.newline);
		xmlString = FileHandler.readFileUTF8(properties.xmlTestFile5);
		msg = XMLAlertParser.getMessageToShow(xmlString);
		System.out.println(msg + properties.newline
				+ "***************************" + properties.newline
				+ properties.newline);

		System.out.println("*** " + properties.xmlTestFile6
				+ properties.newline);
		xmlString = FileHandler.readFileUTF8(properties.xmlTestFile6);
		msg = XMLAlertParser.getMessageToShow(xmlString);
		System.out.println(msg + properties.newline
				+ "***************************" + properties.newline
				+ properties.newline);

		// Document document = XMLAlertParser.getDocument(xmlString, schema);
		// NodeList nodeList = document.getElementsByTagName("point");
		// for (int i=0;i<nodeList.getLength();i++)
		// {
		// Node fstNode = nodeList.item(i);
		// if(fstNode.getNodeType() == Node.ELEMENT_NODE){
		// Element e = (Element) nodeList.item(i);
		// String tagValue = "";
		// System.out.println("Lat : " +getTagValue(e, "latitude"));
		// System.out.println("Nom : " +getTagValue(e, "coordonnees/latitude"));
		// System.out.println("Nom : " +getTagValue(e, "coordonnees/latitude"));
		// }
		// }
		//		
		// String[] keys =
		// {properties.latitudeID,properties.nameID,properties.WaterLevelID};
		// String[] labels =
		// {properties.latLabel,properties.nameLabel,properties.waterLevelLabel};
		// String test = XMLAlertParser.getDataSet(document,
		// "/message/donnees/bulletin_alerte_impact/impact_cote/point",keys,labels
		// );
		// boolean isSeisme = hasElement(document, properties.earthquakeMsg);
		// String id = XMLAlertParser.getValue(document, properties.alertIDkey);
		// String version = XMLAlertParser.getValue(document,
		// properties.alertMessageEarthquake);
	}

	/**
	 * @param e
	 * @param tagValue
	 */
	private static String getTagValue(Element e, String tagValue) {
		String returnValue = "";
		NodeList fstNmElmntLst = e.getElementsByTagName(tagValue);
		Element fstNmElmnt = (Element) fstNmElmntLst.item(0);
		if (fstNmElmnt != null) {
			NodeList fstNm = fstNmElmnt.getChildNodes();
			returnValue = ((Node) fstNm.item(0)).getNodeValue();
		}
		return returnValue;
	}

	/**
	 * Verifies if the given tag exists in the dom document
	 * 
	 * @param document
	 * @param tag
	 * @return
	 */
	private static boolean hasElement(Document document, String tag) {
		if (document != null) {
			NodeList nl = document.getElementsByTagName(tag);
			if (nl.getLength() > 0)
				return true;
		}
		return false;
	}

//	String[] elementsToSearch = { properties.earthquakeMsg,
//			properties.alertManualMsg };

	public static String getMessageToShow(String xmlString) {
		if (xmlString.length() == 0)
			return "";

		String returnValue = "";

		Schema schema = XMLAlertParser.getSchema(properties.xmlSchemaFile);
		Document document = XMLAlertParser.getDocument(xmlString, schema);
 
		for (int i = 0; i < properties.validMessages.length; i++) {
			boolean hasThisMsg = hasElement(document,
					properties.validMessages[i]);
			if (hasThisMsg)
				switch (i) {
				case 0: // earthquakeMsg
					returnValue += properties.earthquakeMsg
							.replaceAll("_", " ")
							+ properties.newline;
					returnValue += properties.messageLabel
							+ XMLAlertParser.getValue(document,
									properties.alertMessageEarthquake);
					break;
				case 1: // alertManualMsg
					returnValue += properties.alertManualMsg.replaceAll("_",
							" ")
							+ properties.newline;
					returnValue += properties.confidenceLevelLabel
					+ XMLAlertParser.getValue(document,
							properties.alertMessageManuelConfidence)+ properties.newline;
					returnValue += properties.messageLabel
							+ XMLAlertParser.getValue(document,
									properties.alertMessageManuelComment);
					break;
				case 2: // alertDetectionMsg
					returnValue += properties.alertDetectionMsg.replaceAll("_",
							" ")
							+ properties.newline;
					returnValue += properties.confidenceLevelLabel
					+ XMLAlertParser.getValue(document,
							properties.detectionMainID+properties.confIndexID)+ properties.newline;
					returnValue += getDataSet(document,
							properties.detectionListID,
							properties.detectionIDs, properties.detectionLabels);
					break;
				case 3: // evolutionMsg
					returnValue += properties.evolutionMsg.replaceAll("_", " ")
							+ properties.newline;
					returnValue += properties.confidenceLevelLabel
					+ XMLAlertParser.getValue(document,
							properties.evolutionMainID+properties.confIndexID)+ properties.newline;
					returnValue += getDataSet(document,
							properties.evolutionListID,
							properties.detectionIDs, properties.detectionLabels);
					break;
				case 4: // impactMsg
					returnValue += properties.impactMsg.replaceAll("_", " ")
							+ properties.newline;
					returnValue += properties.confidenceLevelLabel
					+ XMLAlertParser.getValue(document,
							properties.impactMainID+properties.confIndexID)+ properties.newline;
					returnValue += getDataSet(document,
							properties.impactListID, properties.pointIDs,
							properties.pointLabels);
					break;
				case 5: // waterLevelMsg
					returnValue += properties.waterLevelMsg
							.replaceAll("_", " ")
							+ properties.newline;
					returnValue += properties.confidenceLevelLabel
					+ XMLAlertParser.getValue(document,
							properties.waterMainID+properties.confIndexID)+ properties.newline;
					returnValue += getDataSet(document,
							properties.waterLevelListID, properties.pointIDs,
							properties.pointLabels);
					break;
				case 6: // finDeAlert
					returnValue += "******************** End of The Alert *****************"
							+ properties.newline;
					break;
				default: // default
					returnValue += properties.NoMessageFoundError;
					break;
				}

		}

		String id = XMLAlertParser.getValue(document, properties.alertIDkey);
		if (id != null && id.length() > 0)
			lastMsgID = Integer.parseInt(id);

		return returnValue;

	}

}
